/*
 * @Descripttion: action_test.cpp Project
 * @version: Project v0.1
 * @Author: lenotary
 * @Date: 2021/3/30 下午1:12
 * @LastEditor: lenotary
 * @LastEditTime: 2021/3/30 下午1:12
 */

#include <iostream>
#include <ros/ros.h>
#include <ros/node_handle.h>
#include <termios.h>
#include <csignal>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <sys/poll.h>
#include <thread>
#include <utility>
#include "actionlib/client/simple_action_client.h"
#include "user_msgs/robot_pos_ctrlAction.h"
#include "user_msgs/NAV_POINT.h"
#include "sys/types.h"
#include "unistd.h"
#include "linux/input.h"
#include "fcntl.h"
#include "termio.h"

#define KEYCODE_q 113
#define KEYCODE_c 99
#define KEYCODE_Q 81
#define KEYCODE_SPACE 32
#define KEYCODE_ENTER 10

int scanKeyboard() {
    int in;
    struct termios new_settings;
    struct termios stored_settings;
    tcgetattr(0, &stored_settings);
    new_settings = stored_settings;
    new_settings.c_lflag &= (~ICANON);
    new_settings.c_cc[VTIME] = 0;
    tcgetattr(0, &stored_settings);
    new_settings.c_cc[VMIN] = 1;
    tcsetattr(0, TCSANOW, &new_settings);

    in = getchar();

    tcsetattr(0, TCSANOW, &stored_settings);
    return in;
}

class CActionTest {
public:
    explicit CActionTest(ros::NodeHandle &nh);

private:
    void DoneCb(const actionlib::SimpleClientGoalState &state, const user_msgs::robot_pos_ctrlResultConstPtr &result);

    void ActiveCb();

    void FeedbackCb(const user_msgs::robot_pos_ctrlFeedbackConstPtr &msg);

    std::vector<user_msgs::NAV_POINT> CreatPath(std::vector<int> id, std::vector<int> dir, std::vector<int> type);

    user_msgs::robot_pos_ctrlGoal CreatGoal(std::vector<user_msgs::NAV_POINT> path);

    void KeyGetLoop();

private:
    bool m_bCancelGoal_{false};
    bool m_bIsActive_{false};
    bool m_bPause_{false};
    int m_nGoalNum_{0};
    int m_nGoalNumLast_{0};
    int m_nMaxGoalNum_{4};

    user_msgs::robot_pos_ctrlGoal m_iGoal_;
    actionlib::SimpleActionClient<user_msgs::robot_pos_ctrlAction> m_iActionClient_;
};

CActionTest::CActionTest(ros::NodeHandle &nh) : m_iActionClient_("/robot_pos_ctrl", true) {
    std::vector<int> id0{48, 41};
    std::vector<int> dir0{2, 2};
    std::vector<int> type0{3, 3};

    std::vector<int> id1{41, 1, 2, 3};
    std::vector<int> dir1{2, 2, 2, 2};
    std::vector<int> type1{3, 3, 3, 3};

    std::vector<int> id2{3, 6};
    std::vector<int> dir2{2, 3};
    std::vector<int> type2{3, 3};

    std::vector<int> id3{6, 25, 16};
    std::vector<int> dir3{2, 2, 1};
    std::vector<int> type3{3, 2, 3};

    std::vector<int> id4{16, 21, 22, 21, 16, 25, 6, 3, 2, 1, 41, 48, 49, 48};
    std::vector<int> dir4{2, 1, 2, 2, 3, 3, 2, 1, 2, 2, 2, 2, 2, 2};
    std::vector<int> type4{3, 3, 5, 3, 3, 3, 3, 3, 3, 3, 3, 3, 5, 3};

    auto path0 = CreatPath(id0, dir0, type0);
    auto path1 = CreatPath(id1, dir1, type1);
    auto path2 = CreatPath(id2, dir2, type2);
    auto path3 = CreatPath(id3, dir3, type3);
    auto path4 = CreatPath(id4, dir4, type4);
//    auto path2 = CreatPath(id2, dir2, type2);
    std::thread th1(&CActionTest::KeyGetLoop, this);
    th1.detach();
    ROS_INFO("等待链接 nav action 服务器  robot_pos_ctrl ...");
    bool serverExist = m_iActionClient_.waitForServer(ros::Duration(2.0));
    if (!serverExist) {
        ROS_WARN("服务器连接失败");
        return;
    } else {
        ROS_INFO("服务器链接成功");
    }
    while (ros::ok()) {
        ros::Duration(2.0).sleep();
        if (m_bPause_)
            continue;
        ROS_INFO("目标ID %d", m_nGoalNum_);
        if (m_nGoalNum_ == 0) {
            m_iActionClient_.sendGoal(CreatGoal(path0), [this](auto &&PH1, auto &&PH2) { DoneCb(PH1, PH2); },
                                      [this] { ActiveCb(); }, [this](auto &&PH1) { FeedbackCb(PH1); });
            bool finished_before_timeout = m_iActionClient_.waitForResult(ros::Duration(200));
            if (!finished_before_timeout) {
                ROS_WARN("任务超时 %d", m_nGoalNum_);
                m_iActionClient_.cancelGoal();
                m_nGoalNum_ = -1;
            }
        } else if (m_nGoalNum_ == 1) {
            m_iActionClient_.sendGoal(CreatGoal(path1), [this](auto &&PH1, auto &&PH2) { DoneCb(PH1, PH2); },
                                      [this] { ActiveCb(); }, [this](auto &&PH1) { FeedbackCb(PH1); });
            bool finished_before_timeout = m_iActionClient_.waitForResult(ros::Duration(200));
            if (!finished_before_timeout) {
                ROS_WARN("任务超时 %d", m_nGoalNum_);
                m_iActionClient_.cancelGoal();
                m_nGoalNum_ = -1;
            }
        } else if (m_nGoalNum_ == 2) {
            m_iActionClient_.sendGoal(CreatGoal(path2), [this](auto &&PH1, auto &&PH2) { DoneCb(PH1, PH2); },
                                      [this] { ActiveCb(); }, [this](auto &&PH1) { FeedbackCb(PH1); });
            bool finished_before_timeout = m_iActionClient_.waitForResult(ros::Duration(200));
            if (!finished_before_timeout) {
                ROS_WARN("任务超时 %d", m_nGoalNum_);
                m_iActionClient_.cancelGoal();
                m_nGoalNum_ = -1;
            }
        } else if (m_nGoalNum_ == 3) {
            m_iActionClient_.sendGoal(CreatGoal(path3), [this](auto &&PH1, auto &&PH2) { DoneCb(PH1, PH2); },
                                      [this] { ActiveCb(); }, [this](auto &&PH1) { FeedbackCb(PH1); });
            bool finished_before_timeout = m_iActionClient_.waitForResult(ros::Duration(200));
            if (!finished_before_timeout) {
                ROS_WARN("任务超时 %d", m_nGoalNum_);
                m_iActionClient_.cancelGoal();
                m_nGoalNum_ = -1;
            }
        } else if (m_nGoalNum_ == 4) {
            m_iActionClient_.sendGoal(CreatGoal(path4), [this](auto &&PH1, auto &&PH2) { DoneCb(PH1, PH2); },
                                      [this] { ActiveCb(); }, [this](auto &&PH1) { FeedbackCb(PH1); });
            bool finished_before_timeout = m_iActionClient_.waitForResult(ros::Duration(200));
            if (!finished_before_timeout) {
                ROS_WARN("任务超时 %d", m_nGoalNum_);
                m_iActionClient_.cancelGoal();
                m_nGoalNum_ = -1;
            }
        }
    }
    ROS_INFO("任务取消");
    m_iActionClient_.cancelAllGoals();
}

void CActionTest::DoneCb(const actionlib::SimpleClientGoalState &state,
                         const user_msgs::robot_pos_ctrlResultConstPtr &result) {
    ROS_INFO("任务完成 : %s, %d", result->res.data(), m_nGoalNum_);
    if (result->res == "fail") {
        ros::shutdown();
        return;
    }
    if (m_nGoalNum_ == -1) {
        return;
    }
    m_nGoalNum_++;
    if (m_nGoalNum_ > m_nMaxGoalNum_) {
        m_nGoalNum_ = 0;
    }
}

void CActionTest::ActiveCb() {
    ROS_INFO("goal is active");
    m_bIsActive_ = true;
}

void CActionTest::FeedbackCb(const user_msgs::robot_pos_ctrlFeedbackConstPtr &msg) {
//    ROS_INFO("feedback is %d", msg->drived);
}

std::vector<user_msgs::NAV_POINT>
CActionTest::CreatPath(std::vector<int> id, std::vector<int> dir, std::vector<int> type) {
    std::vector<user_msgs::NAV_POINT> path;

    for (int i = 0; i < id.size(); ++i) {
        user_msgs::NAV_POINT point;
        point.X = id[i];
        point.yaw = dir[i];
        point.type = type[i];
        path.push_back(point);
    }
    return path;
}

user_msgs::robot_pos_ctrlGoal CActionTest::CreatGoal(std::vector<user_msgs::NAV_POINT> path) {
    user_msgs::robot_pos_ctrlGoal goal;
    goal.path = std::move(path);
    return goal;
}

void CActionTest::KeyGetLoop() {
    while (ros::ok()) {
        auto code = scanKeyboard();
        switch (code) {
            case KEYCODE_q:
                if (m_bIsActive_) {
                    ROS_INFO("取消任务");
                    m_iActionClient_.cancelAllGoals();
                    m_nGoalNum_ = -1;
                }
                break;
            case KEYCODE_SPACE:
                if (m_bPause_) {
                    ROS_INFO("继续执行任务");
                } else {
                    ROS_INFO("暂停执行任务");
                }
                break;
            case KEYCODE_c:
                ROS_INFO("开始任务新的任务");
                m_nGoalNum_ = 0;
                break;
            default:
                break;
        }
        ROS_INFO("键盘数值 : %d", scanKeyboard());
    }
}

int main(int argc, char **argv) {
    ros::init(argc, argv, "action_test");
    setlocale(LC_CTYPE, "zh_CN.utf8"); // 设置终端可以输出中文

    ros::NodeHandle nh;
    CActionTest cActionTest(nh);
    return 0;
}