/*
 * @Descripttion: move_action.h Project
 * @version: Project v0.1
 * @Author: lenotary
 * @Date: 2021/4/21 下午1:24
 * @LastEditor: lenotary
 * @LastEditTime: 2021/4/21 下午1:24
 */

#ifndef SRC_MOVE_ACTION_H
#define SRC_MOVE_ACTION_H
#include <iostream>
#include <geometry_msgs/Twist.h>
#include <nav_msgs/Odometry.h>
#include <ros/ros.h>
#include <ros/node_handle.h>
#include <std_msgs/Float32MultiArray.h>
#include "user_msgs/MagLineData.h"

/**
 * @Title:
 * @ClassName: move_action.h .cpp
 * @Description:
 *
 * @Copyright 2016-2021 sunwin - Powered By 研发中心
 * @author: wangpengpeng
 * @date:  2021-04-22 上午11:44
 * @version V1.0
 */

enum MOVE_MODE{
    e_MOVE_ADVANCE,
    e_MOVE_SLOW_DOWN,
    e_MOVE_STOP_SMOOTH,
    e_MOVE_STOP_QUICK,
    e_MOVE_IDLE,
};
/* 枚举类型： 表达运动动作需要的步骤，比如旋转需要分步骤来 */
enum MOVE_STEP{
    e_STEP0,
    e_STEP1,
    e_STEP2,
    e_STEP3,
    e_STEP_END
};
class CMoveAction {
public:
    explicit CMoveAction(ros::NodeHandle &nh);

public:
    void Clear();
    void SetMode(MOVE_MODE mode);
    bool TracingPd (double speed, geometry_msgs::Twist & twist);
    bool Rotation (double angularSpeed, geometry_msgs::Twist & twist);
    bool StopSmooth(geometry_msgs::Twist &twist);
    bool StopQuick(geometry_msgs::Twist &twist);
    bool Advance(double speed, geometry_msgs::Twist & twist,bool useDistance=false);
    bool AdvanceStop(double speed, geometry_msgs::Twist & twist);
    bool ChangeSpeedWithDistance(double speed, double distance, geometry_msgs::Twist &twist);
    bool SetRotationStep(MOVE_STEP moveStep);

private:
    void InitParam();
    void MagLineDataCallback(const user_msgs::MagLineData::ConstPtr &msg);
    void SonarCallback(const std_msgs::Float32MultiArray::ConstPtr &msg);
    void VelCallback(const nav_msgs::Odometry::ConstPtr &msg);
    template <typename T>
    void MyGetParam(std::string paramName, T &param, T defaultValue);
private:


    ros::NodeHandle m_iNh_;
    ros::Subscriber m_iMagLineDataSub_;
    ros::Subscriber m_iVelSub_;

    double m_dSpeedP_{0}; //  speed PID P 参数
    double m_dSpeedD_{0}; //  speed PID D 参数
    double m_dDirP_{0};   // 方向控制 IPD P 参数
    double m_dDirD_{0};   // 方向控制 PID D 参数
    double m_dMaxLineSpeed_{0}; // 速度限制 最大线速度
    double m_dMaxAngularSpeed_{0}; // 角速度限制
    double m_dAngularSpeed_{0}; // 设置圆度旋转速度
    double m_dChangeSpeedDistance_{0}; // 设置设置速度变化距离。通过距离来实现速度平滑
    double m_dStopDistance_{0};
    user_msgs::MagLineData m_iMagLineData_;
    geometry_msgs::Twist m_iVel_;
    std_msgs::Float32MultiArray m_iSonarList_;
    MOVE_STEP m_eTurnAroundStep_{};

    MOVE_MODE m_eMoveMode_{};
    bool m_bChangeSpeeding_{false};
    bool m_bIsStop_{false};
};


#endif //SRC_MOVE_ACTION_H
