/*
 * @Descripttion: navigation.h Project
 * @version: Project v0.1
 * @Author: lenotary
 * @Date: 2021/4/21 下午2:07
 * @LastEditor: lenotary
 * @LastEditTime: 2021/4/21 下午2:07
 */

#ifndef SRC_NAVIGATION_H
#define SRC_NAVIGATION_H

#include <actionlib/server/simple_action_server.h>
#include "user_msgs/robot_pos_ctrlAction.h"
#include "user_msgs/NAV_POINT.h"
#include "user_msgs/RfidMsg.h"
#include "user_msgs/CoproSwitch.h"
#include "user_msgs/MagLineSelect.h"
#include "user_msgs/robot_pos.h"
#include "user_msgs/special_ctrl.h"
#include "user_msgs/SysModeSwitch.h"
#include "move_action.h"
typedef actionlib::SimpleActionServer<user_msgs::robot_pos_ctrlAction> ActionServer;
enum POINT_TYPE{
    e_START_POINT,
    e_CHARGE_POINT,
    e_INTERSECTION_POINT,
    e_STOP_POINT,
    e_END_POINT,
    e_ROTATION_POINT,
    e_OTHER_POINT
};
enum ACTION_STATE{
    e_ACTION_IDLE,
    e_ACTION_RUNNING,
    e_ACTION_ROTATION,
    e_ACTION_PARKING,
    e_ACTION_END,
    e_ACTION_POINT_ERROR,
};
class CNavigation {
public:
    explicit CNavigation(ros::NodeHandle &nh);
    ~CNavigation() = default;

private:
    void ExecuteCb(const user_msgs::robot_pos_ctrlGoalConstPtr& goal);
    void PreemptCb();
    void RfidCallback(const user_msgs::RfidMsg::ConstPtr &msg);
    void CoproSwitchCallback(const user_msgs::CoproSwitch::ConstPtr &msg);
    void SpecialControlCallback(const user_msgs::special_ctrl::ConstPtr &msg);
    void LabelVelCallback(const geometry_msgs::Twist::ConstPtr &msg);
    void ModeCallback(const user_msgs::SysModeSwitch::ConstPtr &msg);
    void Clear();
    void InitParam();
    template <typename T>
    void MyGetParam(std::string paramName, T &param, T defaultValue);
    std::string VectorToStr(std::vector<int>  list);
    bool IsInclude(std::vector<int>  list, int key);
    int GetPointType(int id);
private:
    CMoveAction *m_pfnMoveAction_;
    ros::NodeHandle m_iNh_;
    ros::Publisher m_iVelPub_;  //订阅 发布机器人速度信息
    ros::Publisher m_iMagLineSelectPub_; //发布 磁条选择信息
    ros::Publisher m_iRobotPosePub_; //发布 机器人位置信息
    ros::Subscriber m_iRfidSub_;      // 订阅rfid 信息
    ros::Subscriber m_iCoproSwitchSub_;      // 订阅rfid 信息
    ros::Subscriber m_iSpecialControlSub_;      // 订阅rfid 信息
    ros::Subscriber m_iJoySub_;      // 订阅Joy 信息
    ros::Subscriber m_iLabelSub_;      // 订阅Joy 信息
    ros::Subscriber m_iModeSub_;      // 订阅Joy 信息

    geometry_msgs::Twist m_iJoySpeed_;
    ActionServer m_iActionServer_;  // 动作服务器
    std::vector<user_msgs::NAV_POINT> m_vPath_;
    user_msgs::MagLineSelect m_iLineSelect_;
    user_msgs::SysModeSwitch m_iMode_;
    //设置3档导航速度
    double m_dNavSpeedMax_{0};  //导航时的速度  大
    double m_dNavSpeedMid_{0};  //导航时的速度  中
    double m_dNavSpeedMin_{0};  //导航时的速度  小
    double m_dRotationSpeedMax_{0};  //导航时的速度  小


    //RFID 相关变量
    std::vector<int> m_vPastId_; //经过的ID
    std::vector<int> m_vIdList_; //路径上的RFID，主要做判断机器人移动路径是否超过路径
    int m_nCurrentId_{0};

    //任务相关变量
    int m_nGoalTime_{0}; // 判断路径上终点出现次数
    int m_nIdIndex_{0}; // 判断路径上终点出现次数
    bool m_bParking{false}; // 判断是不是停止
    bool m_bActionActive{true};
    bool m_bIsObstacle_{false};
    ACTION_STATE m_eActionState{};
};


#endif //SRC_NAVIGATION_H
